/*
    This file is part of the Okteta program, made within the KDE community.

    SPDX-FileCopyrightText: 2006-2007 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

#ifndef OKTETAABOUT_HPP
#define OKTETAABOUT_HPP

// KF
#include <KAboutData>

class OktetaAboutData : public KAboutData
{
public:
    OktetaAboutData();
};

#endif
