/*
    This file is part of the Okteta Designer plugin, made within the KDE community.

    SPDX-FileCopyrightText: 2009 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef BYTEARRAYCOLUMNVIEWDESIGNERFACTORY_HPP
#define BYTEARRAYCOLUMNVIEWDESIGNERFACTORY_HPP

// Qt
#include <QDesignerCustomWidgetInterface>
#include <QObject>

class ByteArrayColumnViewDesignerFactory : public QObject
                                         , public QDesignerCustomWidgetInterface
{
    Q_OBJECT
    Q_INTERFACES(
        QDesignerCustomWidgetInterface
    )

public:
    explicit ByteArrayColumnViewDesignerFactory(QObject* parent = nullptr);

public: // QDesignerCustomWidgetInterface API
    QWidget* createWidget(QWidget* parent) override;
    QString group() const override;
    QIcon icon() const override;
    QString includeFile() const override;
    bool isContainer() const override;
    QString name() const override;
    QString toolTip() const override;
    QString whatsThis() const override;
};

#endif
